
% % % % % % % % % % % % % % % % % % % % % % % % % % % %
% Script Goal:
%     connect all graphs components by the biggest edges between them.
%
% INPUTS:
%     ars_matrix: Symmetrical scores matrix for all sequences
%     bool_mat:   boolean matrix of graph edges
%
% OUTPUTS:
%     cut_mat: A final matrix representing a connected graph
% 
% EXAMPLE:
%     ars_mat = [0 5 6 0.5 0.5; 0 0 3 0.5 0.5; 0 0 0 10 0.5; 0 0 0 0 10;0 0 0 0 0];
%     genes_sz = length(ars_mat);
%     ars_mat = ars_mat+ars_mat';
%     ars_mat(1:genes_sz+1:genes_sz*genes_sz)=0;
%     g=graph(ars_mat);
%     figure
%     plot(g,'edgelabel',g.Edges.Weight)
%     bool_mat = (ars_mat > genes_sz*1.5);
%     figure
%     plot(graph(bool_mat))
%     [cut_mat] = keep_graph_connectivity_new(bool_mat,ars_mat);
%     g=graph(cut_mat);
%     figure
%     plot(g,'edgelabel',g.Edges.Weight)
%
% REFERENCE:
%   Galili, M., Tuller, T. (2019) CSN:  Unsupervised Approach for 
%      Inferring Biological Networks based on the Genome Alone. 
% 
% Maya Galili, Apr 2017
% % % % % % % % % % % % % % % % % % % % % % % % % % % %


function [graph_score_matrix] = connect_components_kruskal(bin_mat, score_matrix)

[graph_score_matrix] = load_binary_graph(bin_mat, score_matrix);

[comp_nodes, gcs] = get_set_of_components(graph_score_matrix);

[sorted_max_edges] = find_biggest_edges_between_all_components(comp_nodes, score_matrix);

 [graph_score_matrix] = add_missing_edges(sorted_max_edges, gcs, graph_score_matrix);
end

function [score_matrix] = load_binary_graph(bin_mat, score_matrix)
% Create base graph after the cut-out edges step
genes_sz = length(score_matrix);
score_matrix(1:genes_sz+1:genes_sz*genes_sz)=0;% prevent inner loops
score_matrix = bin_mat.*score_matrix;
end

function [comp_nodes, gcs] = get_set_of_components(score_matrix)
% create set of the different components' nodes
g=graph(score_matrix);
gcs = conncomp(g)';
comp_nodes = arrayfun(@(x) find(gcs==x), unique(gcs),'UniformOutput', false);
end

function [sorted_max_edges] = find_biggest_edges_between_all_components(comp_nodes, score_matrix)
comp_sz = length(comp_nodes);

mat_sz = size(score_matrix);
k=0;

between_comp_sz = comp_sz*(comp_sz-1)/2;
Score = zeros(between_comp_sz, 1);
Node1 = zeros(between_comp_sz, 1);
Node2 = zeros(between_comp_sz, 1);

for i =1:comp_sz-1
    for j = i + 1:comp_sz
        k=k+1;
        comp_mat = zeros(mat_sz);

    % find set of posible edges
        comp1_nodes = comp_nodes{i};
        comp2_nodes = comp_nodes{j};
        comp_mat(comp1_nodes,comp2_nodes)=1;
        comp_mat(comp2_nodes,comp1_nodes)=1;

        comp_mat = comp_mat.*score_matrix;
        node_sz = length(comp_mat);
        comp_mat(1:node_sz+1:node_sz*node_sz)=-inf;

    % find biggest edge
        [Score(k),I] = max(comp_mat(:));
        [Node1(k), Node2(k)] = ind2sub(mat_sz,I);
    end
end

T = table(Score, Node1, Node2);

% sort by score
sorted_max_edges = sortrows(T, 'Score', 'descend');
end


function [graph_score_matrix] = add_missing_edges(sorted_max_edges, gcs, graph_score_matrix)

for i = 1:height(sorted_max_edges)
    score = sorted_max_edges{i,'Score'};
    n1 = sorted_max_edges{i,'Node1'};
    n2 = sorted_max_edges{i,'Node2'};    
    
    first_node_comp = gcs(n1);
    sec_node_comp = gcs(n2);
    
    if first_node_comp ~= sec_node_comp
        first_comp_set = gcs == first_node_comp;
    
        % merge components' nodes
        gcs(first_comp_set) = sec_node_comp;
        graph_score_matrix(n1, n2) = score;
        graph_score_matrix(n2, n1) = score;       
    end
end
end
