% % % % % % % % % % % % % % % % % % % % % % % % % % % %
% Script goal: compute chimeraARS score for each pair
%  - create suffix arrays for all genes.
%  - find the longest common substring starting for every position in the 
%       target gene. 
%  - Calculating the mean length of those substrings. It will give us
%       the ARS score of the target gene with the other gene as reference.
%  - Organize the scores in a single matrix
% 
% INPUTS:
% 	  seq_set: set of N strings
%     is_translate: a flag. determent if we need to calculate score per codons or per nucleotides.
% 
% OUTPUT:
%     score_matrix: NXN scores matrix for all genes paires
% 
% EXAMPLE:
%     seq_set = ['ATTTTGTGTATGAT', 'AGTACCGTATGAT', 'ATTTTGTGTTGGAC'];
%     is_translate = 0;
%     [score_matrix] = get_ars_scores_matrix(seq_set, is_translate); 
% 
% REFERENCES:
%   Galili, M., Tuller, T. (2019) CSN:  Unsupervised Approach for 
%      Inferring Biological Networks based on the Genome Alone. 
% 
% Maya Galili, Apr 2017
% % % % % % % % % % % % % % % % % % % % % % % % % % % %

%% main methode 
function [score_matrix] = get_ars_scores_matrix(seq_set, is_translate)
% create suffix array for each sequence
sa_set = get_sa_set(seq_set, is_translate);

% run ARS scoring
gene_set_sz = length(seq_set);
score_matrix = zeros(gene_set_sz);
for i=1:gene_set_sz
    score_matrix(i,:) = get_scores_vec(i,sa_set,seq_set,is_translate);
end

% remove cross cells
score_matrix(1:gene_set_sz+1:gene_set_sz*gene_set_sz)=0;

% make matrix symetric
score_matrix = (score_matrix+score_matrix')/2;

end

%% create suffix array for all genes
function [sa_set] = get_sa_set(seq_set, by_codons)
sa_set = cellfun(@(x) get_sa(x,by_codons),seq_set,'UniformOutput', false);
end

% create suffix array for one gene
function [suf_idx] = get_sa(S,by_codons)
if by_codons
    S=S(1:end-mod(length(S),3));
    jmp_by = 3;
else
    jmp_by =1;
end
seq_sz = length(S);
[real_SA] = arrayfun(@(x) S(x:end), 1:jmp_by:seq_sz,'UniformOutput', false);

% from string to index
[~, suf_idx] = sort(real_SA);
if by_codons
    suf_idx=(suf_idx.*3.-2)';
end
end

%% chimera ARS scoring
function [ars_vector] = get_scores_vec(gene_id, sa_set, seq_set, by_codons)

% get all genes SA in strings
gene_sa = arrayfun(@(x) from_idx_to_str(seq_set{gene_id}, x),sa_set{gene_id},'UniformOutput', false);

% function output: vector of doubles saved with gene id
ars_vector= cell2mat(cellfun(@(x,y) get_pair_score(gene_sa,x,y),sa_set,seq_set,'UniformOutput', false));

% if we need ars by codons - devide the score by 3.
if(by_codons)
    ars_vector= ars_vector./3;
end
end

%% calculate chimeraARS score for pair of strings 
function [ars_score] =  get_pair_score(this_sa,other_sa,other_seq)
start_idx = 1;
this_gene_sz = length(this_sa);
sbstr_sz_arr = zeros(this_gene_sz,1);
for j=1:this_gene_sz
    % for each one of this gene's suffixes => find common longest other suffix
    tmp_suff = this_sa{j};
    [exists, start_idx]= bin_search(tmp_suff, other_sa,other_seq,start_idx);
    if exists
        sbstr_sz_arr(j) = length(tmp_suff);
        continue;
    end
    
    % the two adjacent suffixes contain the longest common prefix
    neighbor_2 =  from_idx_to_str(other_seq, other_sa(start_idx));
    n2_cc = count_common(tmp_suff, neighbor_2);
    if  start_idx > 1
        neighbor_1= from_idx_to_str(other_seq,other_sa(start_idx-1));
		n1_cc = count_common(tmp_suff, neighbor_1);
        sbstr_sz_arr(j) = max(n1_cc,n2_cc);
    else
        sbstr_sz_arr(j) = n2_cc;
    end
end

ars_score = mean(sbstr_sz_arr);
end

%% binary search - return common substring index
function [in_array, strt_idx] = bin_search(key, other_sa, other_seq, strt_idx)
% init vars
end_idx = length(other_sa);

% loop until start index >= end index
while strt_idx < end_idx
    mid_idx =  floor((strt_idx + end_idx) /2);
    mid_str = from_idx_to_str(other_seq, other_sa(mid_idx));
    
    [~, idx] = sort({key,mid_str});
    if idx(1) > idx(2)
        strt_idx = mid_idx + 1;
    else
        end_idx = mid_idx;
    end
end

in_array = strcmp(from_idx_to_str(other_seq, other_sa(strt_idx)),key);
end

%% return genes SA in strings
function [suff_str] = from_idx_to_str(gene_seq, suf_idx)
suff_str=gene_seq(suf_idx:end);
end

%% find length of common substring
function c = count_common(s1, s2)
% counting the number of first matching characters
n = min(length(s1), length(s2));
c = find(s1(1:n) - s2(1:n), 1, 'first') - 1;
if isempty(c)
    c = n;
end
end