% % % % % % % % % % % % % % % % % % % % % % % % % % % %
% Script Goal:
%     connect all graphs components by the biggest edges between them.
%
% INPUTS:
%     ars_matrix: Symmetrical scores matrix for all sequences
%     bool_mat:   boolean matrix of graph edges
%
% OUTPUTS:
%     cut_mat: A final matrix representing a connected graph
% 
% EXAMPLE:
%     ars_mat = [0 6 5 0.5 0.5; 0 0 3 0.5 0.5; 0 0 0 10 0.5; 0 0 0 0 10;0 0 0 0 0];
%     genes_sz = length(ars_mat);
%     ars_mat = ars_mat+ars_mat';
%     ars_mat(1:genes_sz+1:genes_sz*genes_sz)=0;
%     g=graph(ars_mat);
%     figure
%     plot(g,'edgelabel',g.Edges.Weight)
%     bool_mat = (ars_mat > genes_sz*1.5);
%     figure
%     plot(graph(bool_mat))
%     [cut_mat] = keep_graph_connectivity(bool_mat,ars_mat);
%     g=graph(cut_mat);
%     figure
%     plot(g,'edgelabel',g.edges.weight)
%
% REFERENCE:
%   Galili, M., Tuller, T. (2019) CSN:  Unsupervised Approach for 
%      Inferring Biological Networks based on the Genome Alone. 
% 
% Maya Galili, Apr 2017
% % % % % % % % % % % % % % % % % % % % % % % % % % % %

function [cut_mat] = keep_graph_connectivity(bool_mat,ars_matrix)

% Create an initial graph after the cut-out edges step
genes_sz = length(ars_matrix);
ars_matrix(1:genes_sz+1:genes_sz*genes_sz)=0;% prevent inner loops
cut_mat = bool_mat.*ars_matrix;

% create set of the different components' nodes
g=graph(cut_mat);
gcs = conncomp(g)';
conn_sz = length(unique(gcs));
conn_sets = arrayfun(@(x) find(gcs==x),unique(gcs),'UniformOutput', false);

% mark true for all possible edges in each component
comp_mat = zeros(genes_sz);

for i=1:conn_sz
    tmp_set = conn_sets{i};
    comp_mat(tmp_set,tmp_set)=1;
end

% Connect graph elements by heaviest edges
while conn_sz > 1 % stop condition
   
    % find edges between components
    anti_comp_mat = (~comp_mat).*ars_matrix;
    [M,I] = max(anti_comp_mat(:));
    [first_node, sec_node] = ind2sub(genes_sz,I);
    cut_mat(first_node, sec_node) = M;
    cut_mat(sec_node,first_node) = M;
    
    % find sets of nodes components
    first_node_comp = gcs(first_node);
    sec_node_comp = gcs(sec_node);
    first_comp_set = find(gcs == first_node_comp);
    sec_comp_set =  find(gcs == sec_node_comp);
    
    % merge components' nodes
    gcs(first_comp_set)=sec_node_comp;
    conn_sz = length(unique(gcs));
    
    % true for all possible edges in each component
    comp_mat(first_comp_set,sec_comp_set)=1;
    comp_mat(sec_comp_set,first_comp_set)=1;    
end
end