% % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% FUNCTION GOAL:
%  1. Organize genes in a network graph by their nucleotide or amino acid sequences
%  2. Analyze network features
%
% INPUT:
%  csn_config: Algorithm configuration parameters (SEQ_LENGTH,
%  IS_TRANSLATE, MIN_CORR, ORG_DESCRIPTION)
% 
%  sequence_arr: String array for input sequences.
% 
%  sequence_name_arr: String array for input sequences' names.
%
% OUTPUT:
%  Analysis_results.txt: short network's analysis report
%  CSN_graph.txt: file contain graph edges list - for Cytoscape use
% 
% EXAMPLE:
%     csn_config = struct('SEQ_LENGTH', 5, 'IS_TRANSLATE', 0, 'MIN_CORR', 0.5, 'ORG_DESCRIPTION', 'Random set', 'OUT_DIR', 'C:\res')
%     sequence_arr = ['ATTTTGTGTATGAT', 'AGTACCGTATGAT', 'ATTTTGTGTTGGAC']
%     sequence_name_arr = ['seq1', 'seq2', 'seq3']
%     [graph_matrix] = run_csn(csn_config, sequence_arr, sequence_name_arr)
% 
% REFERENCE:
%   Galili, M., Tuller, T. (2019) CSN:  Unsupervised Approach for 
%      Inferring Biological Networks based on the Genome Alone. 
% 
% Maya Galili, Apr 2017
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

function [graph_matrix] = run_csn(csn_config, sequence_arr, sequence_name_arr)

sequence_arr = cellfun(@(x) x(1:(min(length(x),csn_config.SEQ_LENGTH))),sequence_arr , 'UniformOutput' , false);

% calculate common ARS scores
[score_matrix] = get_ars_scores_matrix(sequence_arr, csn_config.IS_TRANSLATE);

% compute calculation of correlations on the scores matrix
[corr_matrix,~] = corr(score_matrix, 'type', 'Spearman');

% cut off low value edges from the Spearman correlations matrix
bool_mat = (corr_matrix > csn_config.MIN_CORR);
node_sz = length(bool_mat);
bool_mat(1:node_sz+1:node_sz*node_sz)=0;

% keep graph connected
tic
[graph_matrix] = connect_components_kruskal(bool_mat, corr_matrix);
toc

% generate graph's edges list & run graph analysis
save_graph_and_analyze(graph_matrix, sequence_name_arr, csn_config.ORG_DESCRIPTION, csn_config.OUT_DIR);
end