% % % % % % % % % % % % % % % % % % % % % % % % % % % %
% Script goal: conduct first network analysis for the CSN graph
% 
% INPUT:
%  graph_matrix: the CSN graph represented as connectivity matrix
%  name_set: set of the sequences names.
%  fasta_file_nm: a path of FASTA file of genes sequences.
%  
% OUTPUT:
%  CSN_analysis.txt: file with basic analysis of the CSN graph
%  CSN_graph.txt:  edges list (for Cytoscape) 
% 
% REFERENCE:
%   Galili, M., Tuller, T. (2019) CSN:  Unsupervised Approach for 
%      Inferring Biological Networks based on the Genome Alone. 
% 
% Maya Galili, Apr 2017
% % % % % % % % % % % % % % % % % % % % % % % % % % % %

function [] = save_graph_and_analyze(graph_matrix, name_set, fasta_file_nm, res_dir)

% print graph set
G_symm_ars = graph(graph_matrix, name_set);
graph_set = G_symm_ars.Edges;

file_name = fullfile(res_dir, 'CSN_graph.txt');

writetable(graph_set, file_name, 'Delimiter','\t');

% analys the net
simple_net_analysis(G_symm_ars,file_name, fasta_file_nm, res_dir);

% plot the graph if not too big
if length(graph_matrix)<100
    figure;
    plot(G_symm_ars);
end
end

function [] = simple_net_analysis(G_CSN, network_file_nm, fasta_file_nm, res_dir)

% basic properties
number_of_nodes = size(G_CSN.Nodes,1);
number_of_edges = size(G_CSN.Edges,1);
graph_density = number_of_edges/(number_of_nodes*(number_of_nodes-1)/2 );
NodeDegree = degree(G_CSN);
mean_node_degree = mean(NodeDegree);
median_Node_Degree = median(NodeDegree);

% most & less connected genes
GENES_TO_FIND = 10;
[~,I] = sort(NodeDegree);
less_conn_genes = G_CSN.Nodes.Name(I(1:GENES_TO_FIND));
most_conn_genes = G_CSN.Nodes.Name(I(end-GENES_TO_FIND:end));

% save text file with results
file_name = fullfile(res_dir, 'CSN_analysis.txt');
fileID = fopen(file_name,'w');
if fileID > 0
    fprintf(fileID, 'Analyzing %s Common Sub-Sequences Network: \n\n', fasta_file_nm);
    fprintf(fileID, 'Number of nodes:\t\t %.f\n', number_of_nodes );
    fprintf(fileID, 'Number of edges:\t\t %.f\n', number_of_edges);
    fprintf(fileID, 'Graph density:\t\t\t %.4f%% \n', graph_density*100);
    fprintf(fileID, 'Average Node Degree:\t %.4f\n', mean_node_degree);
    fprintf(fileID, 'Median Node Degree:\t\t %.f\n\n', median_Node_Degree);
    fprintf(fileID,'%.f most connected genes: \n', GENES_TO_FIND);
    fprintf(fileID,'%s \n', most_conn_genes{:});
    fprintf(fileID,'\n');
    fprintf(fileID,'%.f least connected genes: \n', GENES_TO_FIND);
    fprintf(fileID,'%s \n', less_conn_genes{:});
    fprintf(fileID,'\n');
    fprintf(fileID, 'For Cytoscape you can use %s file\n',network_file_nm);
    fclose(fileID); 
else
    error('Can not open analysis file!');
end
end